
# MVD 使用说明


# MVD 介绍

**MVD 是一款异构数据库对比工具，可用于异构数据迁移或同步完成之后，校验源库与目标库之间的差异。**

详细使用说明与介绍，请查阅在线文档： [MVD 在线文档](https://docs.mogdb.io/zh/mvd/v3.4/overview)


主要支持如下功能点：

1. **对象结构对比**： 包括源库与目标库的对象差异，表对象的结构（字段类型和名称），约束，索引的差异等。
2. **数据特征对比**： 包括表中各个字段的统计学特征值对比，主要对比相关性与平均值，最大值，最小值，中位值等（效率差，未来考虑废弃）。
3. **精确差异识别**： 对数据特征有差异的表，进行精确差异识别，通过统一的 MD5 摘要算法，找出源库与目标库中数据不一致的行记录。


## 操作系统与平台支持

MVD 目前支持在如下操作系统和平台架构下运行：

1. Linux x86_64
2. Linux arm64    (因为 ARM 平台没有 DB2 客户端，所以 ARM 版不支持源库或者目标库为 DB2 或者 Informix)
3. MacOS


## 数据库支持

MVD 目前支持的源端与目标端数据库类型如下：

1. Oracle >= 11.2
2. MogDB/OpenGauss >= 1.0    ( 暂不支持 SHA256 加密认证方式，推荐对比用户使用 MD5 加密认证 )
3. MySQL >= 5.5
4. DB2 >= 11.5
5. Informix >= 14.10
6. SQL Server >= 16.0

**注意**： 平台支持，与数据库支持没有相关性，数据库可以运行在其他平台，如 Windows/AIX 等，只要从数据库到 MVD 运行主机网络与端口互通即可。


## 逻辑架构

![logical_structure.png](images/logical_structure.png)


# MVD 使用说明

MVD 为纯命令行工具，只能在 Shell 界面中执行。可通过 -h 选项查看命令帮助信息，如下：


```shell

Introduction:
    MVD is a data verification tool for Heterogeneous Databases.

Options:
    -h, --help           : Show help message
    -v, --version        : Show tool version [3.4.1]
    -x, --debug          : Run in debug mode, means more output logs
        --debug-md5      : Debug for print data before calculator MD5
    -c, --config-file    : Using a config file with format json
        --mtk-config     : Using a config file from MTK tool with format json
    -C, --category       : Compare category: A=All, M=Metadata, D=Data
    -d, --func-dimension : [Advanced Option] Functions used in data comparison
                           Default: avg:a,min:np,max:np,median:np
                           Format: <name>:<primary_type>    -- Same function name among all database
                                   <primary_type> := a|p|np, a = all, p = primary table, np = not [p]
                                                     can be ignored, then use default [a]
                           Format: <name>:<function_list>:<primary_type>
                                   <function_list> := <db_type> = <function_name> | <db_type> = <function_name>
                                   <db_type> := ORACLE|ORACLE2|DB2|MYSQL|POSTGRESQL|MOGDB|OPENGAUSS|SQLSERVER|INFORMIX
                           Example 'testmin:oracle=min|mogdb=min|mysql=min|db2=min:p'
    -s, --source-db      : Source database to be verified
                           Format: <db_type>:<ip>:<port>:<name>:<user>:<password>
                                   <db_type> := ORACLE|ORACLE2|DB2|MYSQL|POSTGRESQL|MOGDB|OPENGAUSS|SQLSERVER|INFORMIX
    -t, --target-db      : Target database to be verified, format see also '-s'
        --passwd-encrypt : DB password is encrypted
    -w, --workers        : Parallel workers (1~32), default: 8
    -W, --table-workers  : Parallel workers within one table when partition/sampling parallel used (1~32), default: 4
    -T, --table          : Check a single table (source table info)
                           Format: <owner>.<table_name>
                                   <source_owner>.<source_table>:<target_owner>.<target_table>
                             Tips: 1. if this option was specified, then '-i' and '-e' will be ignored
                                   2. Name with mixed-case means case sensitive
                                   3. Name with quotation mark means keep original case
    -i, --include        : Source White list, patterns used for object filter, all patterns combined with comma
                           Format: <type>:<owner>.<object_name>,...
                                   <type>:<object_name>
                                   <owner>.<object_name>
                                   <object_name>
                             Tips: 1. <type> can use: */%/TABLE/VIEW/SEQUENCE/PROCEDURE/FUNCTION/OTHERS
                                   2. Can use */% in <owner> and <object_name> field, means to match all
                                   3. Name is case insensitive
    -e, --exclude        : Source Black list, patterns used for object filter, format see also '-i'
    -r, --remap-schema   : Schema transformation in comparison
                           Format: <source_schema>:<target_schema>,<source_schema>:<target_schema>...
        --column-list    : Set valid column list for data comparison, combined with comma
                           Format: <colume_item>,<colume_item>,<colume_item>,...
                                   <colume_item> := <source_schema>.<source_name>.<source_colume>:<target_column>
                             Tips: 1. if column name does not changed then ':<target_column>' can be removed
                                   2. if no <source_schema> then it's the limitation for all tables with the same name in each schema
                                   3. if no <source_schema>.<table_name> then it's the limitation for all tables
                                   4. Name with mixed-case means case sensitive
                                   5. Global/Table column mapping/limitation can be used together
                                   6. If global column mapping/limitation used, then all tables columns must in column-list
                                   7. If only table column mapping/limitation used, the column-list scope is only for the table
        --data-filter    : Set data filter for comparison, combined with |
                           Format: <filter_1>|<source_schema>.<source_name>:<filter_source>:<filter_target>,...
                           Example: hongye.test_tab:created > sysdate - 356:created > now() - interval '365 days'
        --sample-size    : Minimal size in MB when using sample comparison (partial data comparison), default 10240 means >= 10GB
        --sample-pct     : Sample percent in sample comparison (value must between 0 and 1), default 1 means compare all data
        --detail-mode    : Result data in detail mode (show data even no differences found)
    -f, --result-file    : Result file, used to save result, default to print result to screen
    -F, --result-format  : Result file data format: json (default), plain
    -R, --row-dir        : Row directory for differences data (MD5 & KEY)
        --row-feedback   : Query row data when differences found, otherwise just key condition listed
        --ignore-float   : Ignore float data type in comparison
        --float-prec     : Float precision in data comparison, Range: 0-6, Default: 4
        --double-prec    : Double precision in data comparison, Range: 0-12, Default: 10
        --fraction-prec  : Fraction precision in data comparison, Range: 0-6, Default: 6 (Informix is 5)
    -z, --zero-char      : Specify a char for chr(0) in comparison, Default is empty char
    -Z, --time-zone      : Specify timezone for DB client, set empty use local, default is UTC(+00:00)
    -l, --logfile        : Write output information to logfile
    -L, --license        : Specify license file, default is: ./license.json
        --apply-license  : Apply for a new license from server
        --upgrade        : Upgrade current binary MVD command
        --callback       : Use callback interface to get PID and result asynchronously

Usage:
    1. Apply a license
       ./mvd_macos_x86_64 --apply-license
    2. Verify a single schema  (Using MD5 ROW-BY-ROW)
       ./mvd_macos_x86_64 -s 'ORACLE:127.0.0.1:1521:orcl:scott:tiger' -t 'MOGDB:127.0.0.1:5432:postgres:hongye:pwd' -i 'HONGYE.*' -R './diff'

```

## 命令选项说明

| 命令选项             | 选项说明                                                     |
| -------------------- | ------------------------------------------------------------ |
| -h, --help           | 显示工具的帮助信息                                           |
| -v, --version        | 显示工具的当前版本信息                                       |
| -x, --debug          | 开启 DEBUG 模式，输出更详细的日志内容，仅在出现问题后的调试过程中使用 |
| --debug-md5          | 专用于 Python MD5 计算前值输出的 debug 选项，该选项会打开 DB2/Oracle 表中的所有数据，占用大量的磁盘日志空间，请谨慎使用 |
| -c, --config-file    | 指定程序运行时的配置参数（json格式），使用配置参数代替具体命令行选项 |
| --mtk-config         | 指定 MTK 运行时的配置文件（json格式），校验 MTK 同步过程中涉及到的对象和表的数据 |
| -C, --catagory       | 指定对比类别： A = 对比所有， M = 仅对比对象结构， D = 仅对比数据 |
| -d, --func-dimension | 高级选项，不是特别清晰该选项含义时，请勿使用。<br />指定数据对比过程中的统计函数维度，输入格式为： <br />   1. &lt;name&gt;:&lt;primary_type&gt;<br />   2. &lt;name&gt;:&lt;function_list&gt;:&lt;primary_type&gt;<br />其中： <br />   primary_type 可选值为 a = 所有表, p = 主键表, np = 非主键表<br />   name 函数名称，若函数名称即为实际数据库中的函数，那么无需指定 function_list，否则需要指定改函数在各个数据库中的实际表现<br />   function_list 指定该函数在各个数据库中的不同表现形式，格式为： &lt;db_type&gt;=&lt;function_name&gt;<br />其中 db_type 可选值如下： ORACLE, DB2, MYSQL, POSTGRESQL, MOGDB, OPENGAUSS, INFORMIX, SQLSERVER |
| -s, --source-db      | 指定对比的源端数据库，其输入格式为： &lt;db_type&gt;:&lt;ip&gt;:&lt;port&gt;:&lt;name&gt;:&lt;user&gt;:&lt;password&gt;<br />其中 db_type 可选值如下： ORACLE, DB2, MYSQL, POSTGRESQL, MOGDB, OPENGAUSS, INFORMIX, SQLSERVER |
| -t, --target-db      | 指定对比的目标端数据库，其输入格式与 `-s` 一样。             |
| --passwd-encrypt     | 数据库密码是否使用 MDB 加密，默认不加密 |
| -w, --workers        | 指定数据对比过程中的并发进程数量，范围是 1~32，默认是 8 个并发 |
| -T, --table          | 仅仅只针对单张表进行对比，输入格式为：&lt;owner&gt;.&lt;table_name&gt; 或 &lt;source_owner&gt;.&lt;source_table&gt;:&lt;target_owner&gt;.&lt;target_table&gt;，不可使用通配符，会与 `-i` 和 `-e` 选项冲突。 此选项支持使用引号（单引号，双引号，反引号）包裹，来保留 schema 和对象名的原始大小写。|
| -i, --include        | 需要包含进行对比的对象列表，可指定多个匹配模式，使用逗号分隔。<br />匹配格式包括：&lt;type&gt;:&lt;owner&gt;.&lt;object_name&gt;, &lt;owner&gt;.&lt;object_name&gt;, &lt;object_name&gt;, ...<br />可以在 OWNER, OBJECT_NAME 中使用 `*` 或 `%` 来标识通配符<br />TYPE 类型包括： */%/TABLE/VIEW/SEQUENCE/PROCEDURE/FUNCTION/OTHERS |
| -e, --exclude        | 需要排除的对象列表，其格式与 `-i` 一样。                     |
| -r, --remap-schema   | 指定对比时的源端与目标端的 Schema 映射关系，默认源端与目标端 Schema 同名无映射 |
| --column-list        | 指定只对比本选项列出的字段数据                              |
| --data-filter        | 指定只对比给定过滤条件的数据                                |
| --sample-size        | 动态采样对比针对的表的大小阈值，大于阈值的表才考虑使用动态采样 |
| --sample-pct         | 动态采样的数据比例，取值 (0, 1] |
| --detail-mode        | 显示详细的数据对比结果，包括无差异表的数据对比结果 |
| -f, --result-file    | 对比结果的输出文件，默认无输出文件，直接在当前执行的命令行窗口展示结果 |
| -F, --result-format  | 对比结果的格式，支持 json, plain 两种，默认 plain 方式，即便于人工阅读的文本格式 |
| -R, --row-dir        | 执行逐行 MD5 对比模式，指定行差异结果的输出文件夹，每张有差异的表，都会在该目录中创建一个差异文件 |
| --row-feedback       | 是否回显差异行的完整数据，默认只显示差异行的 KEY （Oracle 中的 ROWID, PG 中的 CTID 等） |
| --ignore-float       | 是否在数据对比过程中，忽略浮点类型 （float, double, real 等非精确类型），默认不忽略 |
| -z, --zero-char      | 指定 chr(0) 字符在对比中的替代字符，默认为空字符，即移除 chr(0) 不可见字符 |
| -Z, --time-zone      | 指定客户端数据查询的时区，设置空字符串则使用本地操作系统时区，不设置则使用 UTC (+00:00) 时区 |
| -l, --logfile        | 指定工具运行的日志文件                                       |
| -L, --license        | 指定 License 文件的位置，若 license.json 不在当前目录则需要手动通过本选项指定 |
| -l, --apply-license  | 申请 License                                                 |

## 常用命令示例

以下为一些常用场景下的命令示例（以 2.0 MacOS 版本程序为例）。

注意： 特征值统计对比方式可能效率较慢，推荐使用 MD5 逐行对比模式。

0. 申请 License

   ```shell
   ./mvd_macos_x86_64 --apply-license
   ```

   需要在执行过程中输入接收 license 的邮箱地址

1. Compare with a config file

   通过预先编辑好的配置文件，执行对比任务，具体配置文件的配置说明，请参考: [配置文件](./mvd_config.md)

   ```shell
   ./mvd_macos_x86_64 -c config.json
   ```

2. 对比 Oracle 到 MogDB 的结构和数据差异：

   ```shell
   ./mvd_macos_x86_64 -s 'ORACLE:127.0.0.1:1521:orcl:hongye:pwd' -t 'MOGDB:127.0.0.1:5432:omm:hongye:pwd' -i 'mtk.*' -R './diff'
   ```

3. 对比 DB2 到 MogDB 的结构和数据差异：

   ```shell
   ./mvd_macos_x86_64 -s 'DB2:127.0.0.1:50000:HONGYE:db2inst1:pwd' -t 'MOGDB:127.0.0.1:5432:db2_mtk1:hongye:pwd' -i 'mtk.*' -R './diff'
   ```

4. 对比 MySQL 到 MogDB 的结构和数据差异：

   ```shell
   ./mvd_macos_x86_64 -s 'MYSQL:127.0.0.1:3306:hongye:root:pwd' -t 'MOGDB:127.0.0.1:5432:mysql_mtk:hongye:pwd' -i 'mtk.*' -R './diff'
   ```

5. 只对某一张表执行数据对比（特征值对比）：

   ```shell
   ./mvd_macos_x86_64 -s 'ORACLE:127.0.0.1:1521:orcl:scott:tiger' -t 'MOGDB:127.0.0.1:5432:postgres:hongye:pwd' -T 'HONGYE.TEST'
   ```

6. 对比数据和结构，并精确识别差异行：

   ```shell
   ./mvd_macos_x86_64 -s 'ORACLE:127.0.0.1:1521:orcl:scott:tiger' -t 'MOGDB:127.0.0.1:5432:postgres:hongye:pwd' -i 'mtk.*' -R './diff'
   ```

7. 根据 MTK 的配置文件，进行迁移后的数据校验：

   ```shell
   ./mvd_macos_x86_64 --mtk-config oracle2opengauss.json
   ```

8. 使用动态采样对比数据

   动态采样参数 --sample-size 控制表大小超过多少会使用动态采样：100 表示对大小超过 100MB 的表使用动态采样，而 --sample-pct 控制采样比例：0.1 即采样对比 10% 的数据。

   ```shell
   ./mvd_macos_x86_64 -s 'ORACLE:127.0.0.1:1521:orcl:scott:tiger' -t 'MOGDB:127.0.0.1:5432:postgres:hongye:pwd' -t 'mtk.test_big_table' -R './diff' --sample-size 100 --sample-pct 0.1
   ```


# MVD 结果说明

MVD 结果默认以可读文本的方式在终端直接输出，用户可以输出为 json 和 plain 格式，且可以将最终结果输出到指定的结果文件中。以下以文本格式，介绍 MVD 结果各个部分的具体含义。


## 特征值对比

以下结果来源于如下命令：

```shell
./mvd_macos_x86_64 -s 'MYSQL:127.0.0.1:3306::root:pwd' -t 'MOGDB:127.0.0.1:5432:mysql_mtk:hongye:pwd' -i 'mtk.*'
```

对比结果：

```shell
>>>> =============================================
>>>> 1. Environments:
>>>> =============================================
 OPTION               | VALUE                                                   
 ---------------------+-------------------------------------------------------- 
 SOURCE_DB            | ['MYSQL', '121.36.15.2', '55444', '', 'root', '******'] 
 TARGET_DB            | ['MOGDB', '121.36.15.2', '55432', 'mysql_mtk', 'hongye', '******'] 
 WORKERS              | 8                                                       
 INCLUDE              | [['%', 'mtk', '%']]                                     
 RESULT_FORMAT        | json                                                    
 FUNCTION_DIMENSION   | {'AVG': {'TYPE': 'A', 'FUNCTIONS': {'*': 'AVG'}}, 'MIN': {'TYPE': 'NP', 'FUNCTIONS': {'*': 'MIN'}}, 'MAX': {'TYPE': 'NP', 'FUNCTIONS': {'*': 'MAX'}}} 

>>>> =============================================
>>>> 2. Object Differences:
>>>> =============================================
 OBJECT_TYPE               | EXTRA_TYPE      | DIFFERENCE                                                       
 --------------------------+-----------------+------------------------------------------------------------------
 TABLE                     | SOURCE_EXTRA    | ['MTK.TAB_PART_RANGE_HASH', 'MTK.TEST_PK_UK', 'MTK.TAB_PART_KEY_2', 'MTK.TAB_PART_HASH', 'MTK.TAB_PART_RANGE']
 PROCEDURE                 | SOURCE_EXTRA    | ['MTK.TEST_MTK_DATA', 'MTK.TEST_MTK_TEST']                       
 TRIGGER                   | SOURCE_EXTRA    | ['MTK.USER_LOG']                                                 
 SEQUENCE                  | TARGET_EXTRA    | ['MTK.LOGS_ID_SEQ', 'MTK.USERS_ID_SEQ']                          

>>>> =============================================
>>>> 3. Table Structure Differences:
>>>> =============================================
 OWNER | TABLE_NAME         | TYPE             | NAME                      | DIFFERENCE                                                       
 ------+--------------------+------------------+---------------------------+------------------------------------------------------------------
 MTK   | TAB_PART_KEY_1     | CONSTRAINT       | PRIMARY                   | COMMON: []                                                       
 MTK   | TAB_PART_KEY_1     |                  |                           | SOURCE_EXTRA: []                                                 
 MTK   | TAB_PART_KEY_1     |                  |                           | TARGET_EXTRA: ['ID']                                             

>>>> =============================================
>>>> 4. Table Data Differences (Statistics Analysis):
>>>> =============================================
 OWNER | TABLE_NAME | COLUMN_NAME  | DIMENSION |      SOURCE_VALUE |       TARGET_VALUE
 ------+------------+--------------+-----------+-------------------+-------------------
 MTK   | TABLE_NULL | CNT          | ROW       |                 3 |                  0
 MTK   | MTK_TEST_2 | COL_DOUBLE_2 | AVG       | 501.2637835427314 | 501.26378354273165
 MTK   | MTK_TEST_2 | COL_DOUBLE   | AVG       | 499.1979752563913 |  499.1979752563916
 MTK   | MTK_TEST_1 | COL_DOUBLE   | AVG       | 498.8954055279028 |  498.8954055279032
 MTK   | MTK_TEST_4 | COL_DOUBLE_2 | AVG       | 501.5950313959741 |  501.5950313959745
 MTK   | MTK_TEST_4 | COL_DOUBLE   | AVG       | 503.2131632758452 |  503.2131632758458

>>>> =============================================
>>>> 5. Completed
>>>> =============================================
```


## MD5 逐行对比

以下结果来源于如下命令：

```shell
./mvd_macos_x86_64 -s 'MYSQL:127.0.0.1:3306::root:pwd' -t 'MOGDB:127.0.0.1:5432:mysql_mtk:hongye:pwd' -i 'mtk.*' -R './diff'
```

对比结果：

```shell
>>>> =============================================
>>>> 1. Environments:
>>>> =============================================
+--------------------+-------------------------------------------------------------------------------------------------------------------------------------------------------+
| OPTION             | VALUE                                                                                                                                                 |
+--------------------+-------------------------------------------------------------------------------------------------------------------------------------------------------+
| SOURCE_DB          | ['MYSQL', '121.36.15.2', '55444', '', 'root', '******']                                                                                               |
| TARGET_DB          | ['MOGDB', '121.36.15.2', '55432', 'mysql_mtk', 'hongye', '******']                                                                                    |
| WORKERS            | 8                                                                                                                                                     |
| INCLUDE            | [['%', 'MTK', '%']]                                                                                                                                   |
| RESULT_FORMAT      | json                                                                                                                                                  |
| ROW_DIR            | ./diff                                                                                                                                                |
| FUNCTION_DIMENSION | {'AVG': {'TYPE': 'A', 'FUNCTIONS': {'*': 'AVG'}}, 'MIN': {'TYPE': 'NP', 'FUNCTIONS': {'*': 'MIN'}}, 'MAX': {'TYPE': 'NP', 'FUNCTIONS': {'*': 'MAX'}}} |
| CATAGORY           | ALL                                                                                                                                                   |
+--------------------+-------------------------------------------------------------------------------------------------------------------------------------------------------+

>>>> =============================================
>>>> 2. Summary:
>>>> =============================================
+-----------------------+-------+
| OPTION                | VALUE |
+-----------------------+-------+
| TOTAL_OBJECTS         | 22    |
| DIFF_OBJECTS          | 10    |
| TOTAL_TABLES          | 9     |
| STRUCTURE_DIFF_TABLES | 1     |
| DATA_DIFF_TABLES      | 2     |
+-----------------------+-------+

>>>> =============================================
>>>> 3. Object Differences:
>>>> =============================================
+-------------+-------------------+------------------+
| OBJECT_TYPE | SOURCE_EXTRA      | TARGET_EXTRA     |
+-------------+-------------------+------------------+
| TABLE       | MTK.MTK_TEST      |                  |
|             | MTK.MTK_TEST_1    |                  |
|             | MTK.MTK_TEST_2    |                  |
|             | MTK.MTK_TEST_3    |                  |
|             | MTK.MTK_TEST_4    |                  |
| TRIGGER     | MTK.USER_LOG      |                  |
| SEQUENCE    |                   | MTK.SEQ_LOGS_ID  |
|             |                   | MTK.SEQ_USERS_ID |
| PROCEDURE   | MTK.TEST_MTK_DATA |                  |
|             | MTK.TEST_MTK_TEST |                  |
+-------------+-------------------+------------------+

>>>> =============================================
>>>> 4. Table Structure Differences:
>>>> =============================================
+-------+---------------+-----------------+---------+--------+--------+
| OWNER | TABLE         | TYPE            | NAME    | SOURCE | TARGET |
+-------+---------------+-----------------+---------+--------+--------+
| MTK   | TEST_VARCHAR3 | COLUMN_NULLABLE | ID      | Y      | N      |
| MTK   | TEST_VARCHAR3 | CONSTRAINT      | PRIMARY |        | ID     |
| MTK   | TEST_VARCHAR3 | INDEX           |         |        | ID     |
+-------+---------------+-----------------+---------+--------+--------+

>>>> =============================================
>>>> 5. Table Data Differences (MD5 ROW-BY-ROW):
>>>> =============================================
+-------+---------------+--------|-------------+-------------+-------------+-------------+
| OWNER | TABLE         | MODE   | SOURCE_ROWS | TARGET_ROWS | SOURCE_DIFF | TARGET_DIFF |
+-------+---------------+--------+-------------+-------------+-------------+-------------+
| MTK   | TABLE_NULL    | Normal |           3 |           3 |           2 |           2 |
| MTK   | TEST_VARCHAR3 | Normal |           6 |           6 |           1 |           1 |
+-------+---------------+--------+-------------+-------------+-------------+-------------+

>>>> =============================================
>>>> 6. Completed
>>>> =============================================
```


## Environments

环境参数与变量信息，展示当前运行过程中的一些程序参数与变量。


## Object Differences

对象差异，按类别汇总源库和目标库的对象差异，该结果分 3 列展示：

- **OBJECT_TYPE**: 差异对象的类别
- **SOURCE_EXTRA**: 源库中多出来的对象，以列表形式展示，每行为一个对象，列表按名称排序
- **TARGET_EXTRA**: 目标库中多出来的对象，以列表形式展示，每行为一个对象，列表按名称排序


## Table Structure Differences

表结构差异，展示每一张差异表的具体差异项，该结果分为 5 列展示：

- **OWNER**: 表的 Schema 信息
- **TABLE_NAME**: 表名
- **TYPE**: 差异类别，主要有： COLUMN_COUNT, COLUMN, COLUMN_POSITION, COLUMN_DATATYPE, COLUMN_NULLABLE, CONSTRAINT, INDEX 等
- **NAME**: 差异名称，不同差异类别中，差异名称有不同的含义
- **SOURCE**: 源库中的差异值，不同类别下差异值是不一样的
- **TARGET**: 目标库中的差异值，不同类别下差异值是不一样的

各差异类别下，差异名称与差异项含义说明如下：

| 差异类别        | 差异名称 | 差异项                                                       |
| --------------- | -------- | ------------------------------------------------------------ |
| COLUMN_COUNT    | 无       | 格式： SOURCE: {n}, TARGET: {m}<br />其中： n, m 分别表示源库和目标库中字段的数量 |
| COLUMN          | 无       | 格式：COMMON: {x} SOURCE_EXTRA: {y} TARGET_EXTRA: {z}<br />其中：x 表示源库和目标库中都存在的字段，y 表示源库多余的字段，z 表示目标库多余的字段 |
| COLUMN_POSITION | 字段名   | 格式： SOURCE: {n}, TARGET: {m}<br />其中： n, m 分别表示对应的字段在源库和目标库中的位置信息 |
| COLUMN_DATATYPE | 字段名   | 格式： SOURCE: {n}, TARGET: {m}<br />其中： n, m 分别表示对应的字段在源库和目标库中的类型信息 |
| COLUMN_NULLABLE | 字段名   | 格式： SOURCE: {n}, TARGET: {m}<br />其中： n, m 分别表示对应的字段在源库和目标库中的是否为空信息 |
| CONSTRAINT      | 约束类型 | 格式：COMMON: {x} SOURCE_EXTRA: {y} TARGET_EXTRA: {z}<br />其中：x 表示源库和目标库中都存在的约束，y 表示源库多余的约束，z 表示目标库多余的约束 |
| INDEX           | 无       | 格式：COMMON: {x} SOURCE_EXTRA: {y} TARGET_EXTRA: {z}<br />其中：x 表示源库和目标库中都存在的索引，y 表示源库多余的索引，z 表示目标库多余的索引 |

**注意事项**：

- 对于约束和索引，名称不同不认为存在差异，只要约束和索引对应的 “字段名称，位置，顺序，是否使用函数” 等信息一致，即认为双方是一致的。即如下两个索引认为是相同的：

```sql
-- Oracle
create index idx_test_pk_oracle on hongye.test(id);
-- MogDB
create index idx_test_pk_mogdb on hongye.test(id);
```


## Table Data Differences

数据差异，展示表中的数据差异，无差异的表表不展示。数据差异按照对比方式的不同分为两种： 

- 特征值对比（Statistics Analysis）

- 逐行对比（MD5 ROW-BY-ROW）

特征值对比结果分为 6 列展示：

- **OWNER**: 表的 Schema 信息
- **TABLE_NAME**: 表名
- **COLUMN_NAME**: 字段名称
- **DIMENSION**: 统计维度，默认情况下，对于主键表，统计维度为：COUNT + AVG + CORR，对于非主键表，统计维度为： COUNT + AVG + MIN + MAX + MEDIAN
- **SOURCE_VALUE**: 源库统计值
- **TARGET_VALUE**: 目标库统计值

逐行对比结果分为 7 列展示：

- **OWNER**: 表的 Schema 信息
- **NAME**: 表名
- **MODE**: 对比模式，分为：Normal, Partition, Sample, Split, Partial
- **SOURCE_ROWS**: 源端总行数
- **TARGET_ROWS**: 目标端总行数
- **SOURCE_DIFF**: 源端差异行数 （即源端存在且与目标端不一致的行的数量）
- **TARGET_DIFF**: 目标端差异行数 （即目标端存在且与源端不一致的行的数量）
